@@ -12,6 +12,7 @@ import uproot
 order = [
     "uproot",
     "uproot.reading",
+    "uproot.writing",
     "uproot.behaviors",
     "uproot.behavior",
     "uproot.model",
@@ -19,7 +20,10 @@ order = [
     "uproot.cache",
     "uproot.compression",
     "uproot.deserialization",
+    "uproot.serialization",
+    "uproot.pyroot",
     "uproot.source",
+    "uproot.sink",
     "uproot.interpretation",
     "uproot.containers",
     "uproot.language",
@@ -27,16 +31,59 @@ order = [
     "uproot.exceptions",
 ]
 
+common = [
+    "uproot.reading.open",
+    "uproot.behaviors.TBranch.iterate",
+    "uproot.behaviors.TBranch.concatenate",
+    "uproot.behaviors.TBranch.lazy",
+    "uproot.writing.writable.create",
+    "uproot.writing.writable.recreate",
+    "uproot.writing.writable.update",
+    "uproot.reading.ReadOnlyFile",
+    "uproot.reading.ReadOnlyDirectory",
+    "uproot.behaviors.TTree.TTree",
+    "uproot.behaviors.TBranch.TBranch",
+    "uproot.writing.writable.WritableFile",
+    "uproot.writing.writable.WritableDirectory",
+    "uproot.writing.writable.WritableTree",
+    "uproot.writing.writable.WritableBranch",
+    "uproot.compression.ZLIB",
+    "uproot.compression.LZMA",
+    "uproot.compression.LZ4",
+    "uproot.compression.ZSTD",
+    "uproot.cache.LRUCache",
+    "uproot.cache.LRUArrayCache",
+    "uproot.model.Model",
+    "uproot.pyroot.from_pyroot",
+    "uproot.source.object.ObjectSource",
+    "uproot.source.file.MemmapSource",
+    "uproot.source.file.MultithreadedFileSource",
+    "uproot.source.http.HTTPSource",
+    "uproot.source.http.MultithreadedHTTPSource",
+    "uproot.source.xrootd.XRootDSource",
+    "uproot.source.xrootd.MultithreadedXRootDSource",
+]
+
 latest_commit = (
     subprocess.run(["git", "rev-parse", "HEAD"], stdout=subprocess.PIPE)
     .stdout.decode("utf-8")
     .strip()
 )
 
+main = open("main.toctree", "w")
+main.write(
+    """.. toctree::
+    :caption: Main Interface
+    :hidden:
+
+{0}""".format(
+        "".join("    {0}\n".format(x) for x in common)
+    )
+)
 toctree = open("uproot.toctree", "w")
 toctree.write(
     """.. toctree::
-    :caption: Reference
+    :caption: Detailed Reference
     :hidden:
 
 """
@@ -56,6 +103,9 @@ def ensure(filename, content):
 
 
 def handle_module(modulename, module):
+    if any(x.startswith("_") for x in modulename.split(".")):
+        return
+
     content = """{0}
 {1}
 
@@ -65,9 +115,9 @@ def handle_module(modulename, module):
     )
     ensure(modulename + ".rst", content)
     if toctree2 is None:
-        toctree.write("    " + modulename + "\n")
+        toctree.write("    " + modulename + " (module) <" + modulename + ">\n")
     else:
-        toctree2.write("    " + modulename + "\n")
+        toctree2.write("    " + modulename + " (module) <" + modulename + ">\n")
 
     if modulename != "uproot" and all(
         not x.startswith("_") for x in modulename.split(".")
@@ -81,7 +131,10 @@ def handle_module(modulename, module):
 
         def line_order(pair):
             name, obj = pair
-            return inspect.getsourcelines(obj)[1]
+            try:
+                return inspect.getsourcelines(obj)[1]
+            except OSError:
+                return float("inf")
 
         for pair in sorted(inspect.getmembers(module, good), key=line_order):
             name, obj = pair
@@ -115,7 +168,10 @@ def handle_class(classname, cls):
                 if inspect.ismethod(obj) or inspect.isfunction(obj):
                     module, name = obj.__module__, obj.__name__
                 elif isinstance(obj, property):
-                    module, name = obj.fget.__module__, obj.fget.__name__
+                    try:
+                        module, name = obj.fget.__module__, obj.fget.__name__
+                    except AttributeError:
+                        return False
                 else:
                     module, name = "", ""
                 if module.startswith("uproot."):
@@ -157,14 +213,15 @@ def handle_class(classname, cls):
 
     fullfilename = importlib.import_module(cls.__module__).__file__
     shortfilename = fullfilename[fullfilename.rindex("uproot/") :]
-    link = "`{0} <https://github.com/scikit-hep/uproot4/blob/{1}/{2}>`__".format(
+    link = "`{0} <https://github.com/scikit-hep/uproot4/blob/{1}/src/{2}>`__".format(
         cls.__module__, latest_commit, shortfilename
     )
-    linelink = (
-        "`line {0} <https://github.com/scikit-hep/uproot4/blob/{1}/{2}#L{0}>`__".format(
+    try:
+        linelink = "`line {0} <https://github.com/scikit-hep/uproot4/blob/{1}/src/{2}#L{0}>`__".format(
             inspect.getsourcelines(cls)[1], latest_commit, shortfilename
         )
-    )
+    except OSError:
+        linelink = ""
 
     inheritance_header = ""
     inheritance_footer = ""
@@ -211,7 +268,8 @@ Defined in {2} on {3}.
 
     ensure(classname + ".rst", content)
     if upfront or toctree2 is None:
-        toctree.write("    " + classname + "\n")
+        if classname not in common:
+            toctree.write("    " + classname + "\n")
         toctree2.write("    " + classname + " <" + classname + ">\n")
     else:
         toctree2.write("    " + classname + "\n")
@@ -227,13 +285,11 @@ def handle_function(functionname, cls):
 
     fullfilename = importlib.import_module(cls.__module__).__file__
     shortfilename = fullfilename[fullfilename.rindex("uproot/") :]
-    link = "`{0} <https://github.com/scikit-hep/uproot4/blob/{1}/{2}>`__".format(
+    link = "`{0} <https://github.com/scikit-hep/uproot4/blob/{1}/src/{2}>`__".format(
         cls.__module__, latest_commit, shortfilename
     )
-    linelink = (
-        "`line {0} <https://github.com/scikit-hep/uproot4/blob/{1}/{2}#L{0}>`__".format(
-            inspect.getsourcelines(cls)[1], latest_commit, shortfilename
-        )
+    linelink = "`line {0} <https://github.com/scikit-hep/uproot4/blob/{1}/src/{2}#L{0}>`__".format(
+        inspect.getsourcelines(cls)[1], latest_commit, shortfilename
     )
 
     content = """{0}
@@ -247,7 +303,8 @@ Defined in {2} on {3}.
     )
     ensure(functionname + ".rst", content)
     if upfront or toctree2 is None:
-        toctree.write("    " + functionname + "\n")
+        if functionname not in common:
+            toctree.write("    " + functionname + "\n")
         toctree2.write("    " + functionname + " <" + functionname + ">\n")
     else:
         toctree2.write("    " + functionname + "\n")
