@@ -620,16 +620,30 @@ in object {3}""".format(
             for name, context in expression_context:
                 array = renamed_arrays[_rename(name, context)] = arrays[name]
                 if context["is_jagged"]:
+                    if (
+                        isinstance(
+                            array.layout,
+                            (
+                                awkward.layout.ListArray32,
+                                awkward.layout.ListArrayU32,
+                                awkward.layout.ListArray64,
+                            ),
+                        )
+                        or array.layout.offsets[0] != 0
+                    ):
+                        array_layout = array.layout.toListOffsetArray64(True)
+                    else:
+                        array_layout = array.layout
                     if len(offsets) == 0:
-                        offsets.append(array.layout.offsets)
+                        offsets.append(array_layout.offsets)
                         jaggeds.append([_rename(name, context)])
                     else:
                         for o, j in zip(offsets, jaggeds):
-                            if numpy.array_equal(array.layout.offsets, o):
+                            if numpy.array_equal(array_layout.offsets, o):
                                 j.append(_rename(name, context))
                                 break
                         else:
-                            offsets.append(array.layout.offsets)
+                            offsets.append(array_layout.offsets)
                             jaggeds.append([_rename(name, context)])
                 else:
                     nonjagged.append(_rename(name, context))
@@ -653,6 +667,12 @@ in object {3}""".format(
                         cut -= 1
                     if cut == 0:
                         break
+                if (
+                    out is not None
+                    and cut != 0
+                    and jagged[0][:cut].strip("_./") in awkward.fields(out)
+                ):
+                    cut = 0
                 if cut == 0:
                     common = "jagged{0}".format(number)
                     if len(jagged) == 0:
@@ -679,8 +699,8 @@ in object {3}""".format(
                 if out is None:
                     out = awkward.Array({common: subarray})
                 else:
-                    for _ in jagged:
-                        out = awkward.with_field(out, subarray, common)
+                    out = awkward.with_field(out, subarray, common)
+
             return out
         else:
             raise TypeError(
@@ -975,6 +995,32 @@ class Pandas(Library):
                                 flat_index = pandas.MultiIndex.from_arrays(
                                     [array.index]
                                 )
+                            # Old versions of Pandas handle the following line poorly:
+                            # should we support them?
+                            #
+                            # >>> pandas.__version__
+                            # '0.22.0'
+                            # >>> from_index = pandas.MultiIndex.from_tuples([(0,), (1,)])
+                            # >>> to_index = pandas.MultiIndex.from_tuples([(0, 0), (0, 1), (0, 2), (1, 0), (1, 1)])
+                            # >>> pandas.Series([1.1, 4.4], index=from_index).reindex(to_index)
+                            # 0  0   NaN
+                            #    1   NaN
+                            #    2   NaN
+                            # 1  0   NaN
+                            #    1   NaN
+                            # dtype: float64
+                            #
+                            # >>> pandas.__version__
+                            # '1.3.2'
+                            # >>> from_index = pandas.MultiIndex.from_tuples([(0,), (1,)])
+                            # >>> to_index = pandas.MultiIndex.from_tuples([(0, 0), (0, 1), (0, 2), (1, 0), (1, 1)])
+                            # >>> pandas.Series([1.1, 4.4], index=from_index).reindex(to_index)
+                            # 0  0    1.1
+                            #    1    1.1
+                            #    2    1.1
+                            # 1  0    4.4
+                            #    1    4.4
+                            # dtype: float64
                             df.append(
                                 pandas.Series(array.values, index=flat_index).reindex(
                                     index
@@ -1039,6 +1085,9 @@ class Pandas(Library):
             )
 
     def global_index(self, arrays, global_offset):
+        if isinstance(arrays, tuple):
+            return tuple(self.global_index(x, global_offset) for x in arrays)
+
         if type(arrays.index).__name__ == "MultiIndex":
             if hasattr(arrays.index.levels[0], "arrays"):
                 index = arrays.index.levels[0].arrays  # pandas>=0.24.0
