@@ -46,6 +46,10 @@ bootstrap_classnames = [
     "TObjArray",
     "TObjString",
 ]
+never_from_streamers = [
+    "TString",
+    "TList",
+]
 
 np_uint8 = numpy.dtype("u1")
 
@@ -90,12 +94,17 @@ def reset_classes():
     reload(uproot.models.TObjArray)
     reload(uproot.models.TObjString)
     reload(uproot.models.TAtt)
+    reload(uproot.models.TDatime)
     reload(uproot.models.TRef)
+
+    reload(uproot.models.TTable)
     reload(uproot.models.TTree)
     reload(uproot.models.TBranch)
     reload(uproot.models.TLeaf)
     reload(uproot.models.TBasket)
     reload(uproot.models.RNTuple)
+    reload(uproot.models.TH)
+    reload(uproot.models.TGraph)
 
 
 _classname_regularize = re.compile(r"\s*(<|>|::)\s*")
@@ -282,7 +291,7 @@ def has_class_named(classname, version=None, custom_classes=None):
     Returns True if :doc:`uproot.model.class_named` would find a class,
     False if it would raise an exception.
     """
-    cls = maybe_custom_classes(custom_classes).get(classname)
+    cls = maybe_custom_classes(classname, custom_classes).get(classname)
     if cls is None:
         return False
 
@@ -292,12 +301,14 @@ def has_class_named(classname, version=None, custom_classes=None):
         return True
 
 
-def maybe_custom_classes(custom_classes):
+def maybe_custom_classes(classname, custom_classes):
     """
     Passes through ``custom_classes`` if it is not None; returns
     ``uproot.classes`` otherwise.
+
+    Some ``classnames`` are never custom (see ``uproot.model.never_from_streamers``).
     """
-    if custom_classes is None:
+    if custom_classes is None or classname in never_from_streamers:
         return uproot.classes
     else:
         return custom_classes
@@ -361,6 +372,7 @@ class Model(object):
     class_streamer = None
     class_rawstreamers = ()
     writable = False
+    _deeply_writable = False
     behaviors = ()
 
     def __repr__(self):
@@ -721,6 +733,7 @@ class Model(object):
         self._cursor = None
         self._file = None
         self._parent = None
+        self._concrete = None
         self._members = {}
         self._bases = []
         self._num_bytes = None
@@ -757,7 +770,6 @@ class Model(object):
         self._file = selffile
         self._parent = parent
         self._concrete = concrete
-
         self._members = {}
         self._bases = []
         self._num_bytes = None
@@ -956,9 +968,75 @@ class Model(object):
         """
         pass
 
-    def _serialize(self, out, header, name):
+    def _to_writable_postprocess(self, original):
+        pass
+
+    def _to_writable(self, concrete):
+        cls = None
+        if self.writable:
+            cls = type(self)
+        else:
+            unversioned = uproot.classes.get(self.classname)
+            if issubclass(unversioned, DispatchByVersion):
+                for versioned_cls in unversioned.known_versions.values():
+                    if versioned_cls.writable:
+                        cls = versioned_cls
+                        break
+
+            elif unversioned is not None:
+                if unversioned.writable:
+                    cls = unversioned
+
+        if cls is None:
+            raise NotImplementedError(
+                "this ROOT type is not writable: {0}".format(self.classname)
+            )
+        else:
+            out = cls.__new__(cls)
+            out._cursor = self._cursor
+            out._file = self._file
+            out._parent = self._parent
+            out._concrete = concrete
+            out._num_bytes = self._num_bytes
+            out._instance_version = classname_decode(cls.__name__)[1]
+            out._is_memberwise = self._is_memberwise
+
+            if concrete is None:
+                concrete = out
+
+            out._bases = []
+            for base in self._bases:
+                out._bases.append(base._to_writable(concrete))
+
+            out._members = {}
+            for key, value in self._members.items():
+                if isinstance(value, Model):
+                    out._members[key] = value._to_writable(None)
+                else:
+                    out._members[key] = value
+
+            out._to_writable_postprocess(self)
+            return out
+
+    def to_writable(self):
+        """
+        Args:
+            obj (:doc:`uproot.model.Model` instance of the same C++ class): The
+                object to convert to this class version.
+
+        Returns a writable version of this object or raises a NotImplementedError
+        if no writable version exists.
+        """
+        if self._deeply_writable:
+            return self
+        else:
+            return self._to_writable(None)
+
+    def _serialize(self, out, header, name, tobject_flags):
         raise NotImplementedError(
-            "serialize method not implemented on {0}".format(type(self).__name__)
+            "can't write {0} instances yet ('serialize' method not implemented)".format(
+                type(self).__name__
+            )
         )
 
     def serialize(self, name=None):
@@ -972,9 +1050,21 @@ class Model(object):
         NotImplementedError).
         """
         out = []
-        self._serialize(out, True, name)
+        self._serialize(out, True, name, numpy.uint32(0x00000000))
         return b"".join(out)
 
+    def to_pyroot(self, name=None):
+        """
+        Args:
+            name (str or None): A name for the new PyROOT object.
+
+        Converts this :doc:`uproot.model.Model` into a PyROOT object *if it is writable*.
+        A minority of Uproot models are writable, mostly just histograms. Writability
+        is necessary for conversion to PyROOT because it is serialized through a
+        ROOT TMessage.
+        """
+        return uproot.pyroot.to_pyroot(self, name=name)
+
 
 class VersionedModel(Model):
     """
@@ -1128,6 +1218,12 @@ class DispatchByVersion(object):
         if streamer is None:
             streamer = file.streamer_named(classname, "max")
 
+        if streamer is None and file.custom_classes is not None:
+            cls2 = uproot.classes.get(classname)
+            versioned_cls2 = cls2.class_of_version(version)
+            if versioned_cls2 is not None:
+                return versioned_cls2
+
         if streamer is not None:
             versioned_cls = streamer.new_class(file)
             versioned_cls.class_streamer = streamer
