@@ -162,28 +162,40 @@ def open(
         return file.root_directory[object_path]
 
 
-open.defaults = {
-    "file_handler": uproot.source.file.MemmapSource,
-    "xrootd_handler": uproot.source.xrootd.XRootDSource,
-    "http_handler": uproot.source.http.HTTPSource,
-    "object_handler": uproot.source.object.ObjectSource,
-    "timeout": 30,
-    "max_num_elements": None,
-    "num_workers": 1,
-    "num_fallback_workers": 10,
-    "begin_chunk_size": 403,  # the smallest a ROOT file can be
-    "minimal_ttree_metadata": True,
-}
-# See https://github.com/scikit-hep/uproot4/issues/294
-if uproot.extras.older_xrootd("5.2.0"):
-    dist = pkg_resources.get_distribution("XRootD")
-    message = """XRootD {0} is not fully supported; either upgrade to 5.2.0+ or set
+class _OpenDefaults(dict):
+    def __getitem__(self, where):
+        if where == "xrootd_handler" and where not in self:
+            # See https://github.com/scikit-hep/uproot4/issues/294
+            if uproot.extras.older_xrootd("5.2.0"):
+                dist = pkg_resources.get_distribution("XRootD")
+                message = (
+                    "XRootD {0} is not fully supported; ".format(dist.version)
+                    + """either upgrade to 5.2.0+ or set
 
     open.defaults["xrootd_handler"] = uproot.MultithreadedXRootDSource
-""".format(
-        dist.version
-    )
-    warnings.warn(message, FutureWarning)
+"""
+                )
+                warnings.warn(message, FutureWarning)
+
+            # The key should still be set, regardless of whether we see the warning.
+            self["xrootd_handler"] = uproot.source.xrootd.XRootDSource
+
+        return dict.__getitem__(self, where)
+
+
+open.defaults = _OpenDefaults(
+    {
+        "file_handler": uproot.source.file.MemmapSource,
+        "http_handler": uproot.source.http.HTTPSource,
+        "object_handler": uproot.source.object.ObjectSource,
+        "timeout": 30,
+        "max_num_elements": None,
+        "num_workers": 1,
+        "num_fallback_workers": 10,
+        "begin_chunk_size": 403,  # the smallest a ROOT file can be
+        "minimal_ttree_metadata": True,
+    }
+)
 
 
 must_be_attached = [
@@ -231,7 +243,7 @@ class CommonFileMethods(object):
         """
         The dict of ``options`` originally passed to the file constructor.
 
-        If this is a :doc:`uproot.writing.WritableFile`, the ``options`` are a copy
+        If this is a :doc:`uproot.writing.writable.WritableFile`, the ``options`` are a copy
         of the current state of the options; change the properties (e.g.
         ``initial_directory_bytes``, ``uuid_function``) directly on the file object
         to make a lasting change. Modifying the copied dict does not change the
@@ -389,9 +401,8 @@ class CommonFileMethods(object):
     @property
     def fUnits(self):
         """
-        Number of bytes in the serialization of file seek points.
-
-        Usually 4 or 8.
+        Number of bytes in the serialization of file seek points, which can either
+        be 4 or 8.
         """
         return self._fUnits
 
@@ -562,7 +573,7 @@ class ReadOnlyFile(CommonFileMethods):
         self.decompression_executor = decompression_executor
         self.interpretation_executor = interpretation_executor
 
-        self._options = dict(open.defaults)
+        self._options = _OpenDefaults(open.defaults)
         self._options.update(options)
         for option in ["begin_chunk_size"]:
             self._options[option] = uproot._util.memory_size(self._options[option])
@@ -890,7 +901,9 @@ in file {1}""".format(
                     streamer_chunk=streamer_chunk,
                 )
 
-                classes = uproot.model.maybe_custom_classes(self._custom_classes)
+                classes = uproot.model.maybe_custom_classes(
+                    "TList", self._custom_classes
+                )
                 tlist = classes["TList"].read(
                     streamer_chunk, streamer_cursor, {}, self, self.detached, None
                 )
@@ -1117,12 +1130,15 @@ in file {1}""".format(
         global ``uproot.classes``.
         """
         classname = uproot.model.classname_regularize(classname)
-        classes = uproot.model.maybe_custom_classes(self._custom_classes)
+        classes = uproot.model.maybe_custom_classes(classname, self._custom_classes)
         cls = classes.get(classname)
 
         if cls is None:
             streamers = self.streamers_named(classname)
+            if len(streamers) == 0 and self._custom_classes is not None:
+                cls = uproot.classes.get(classname)
 
+        if cls is None:
             if len(streamers) == 0:
                 unknown_cls = uproot.unknown_classes.get(classname)
                 if unknown_cls is None:
@@ -1517,6 +1533,26 @@ class ReadOnlyDirectory(Mapping):
     def __exit__(self, exception_type, exception_value, traceback):
         self._file.source.__exit__(exception_type, exception_value, traceback)
 
+    @property
+    def created_on(self):
+        """
+        Creation date/time as a Python datetime.
+
+        :ref:`uproot.reading.ReadOnlyDirectory.fDatimeC` presents this in ROOT's
+        raw integer encoding.
+        """
+        return uproot._util.code_to_datetime(self._fDatimeC)
+
+    @property
+    def modified_on(self):
+        """
+        Modification date/time as a Python datetime.
+
+        :ref:`uproot.reading.ReadOnlyDirectory.fDatimeM` presents this in ROOT's
+        raw integer encoding.
+        """
+        return uproot._util.code_to_datetime(self._fDatimeM)
+
     @property
     def cursor(self):
         """
@@ -2033,7 +2069,7 @@ class ReadOnlyDirectory(Mapping):
                     else:
                         raise uproot.KeyInFileError(
                             where,
-                            because=repr(item)
+                            because="/".join(items[:i])
                             + " is not a TDirectory, TTree, or TBranch",
                             keys=[key.fName for key in last._keys],
                             file_path=self._file.file_path,
@@ -2056,7 +2092,8 @@ class ReadOnlyDirectory(Mapping):
         """
         Raw integer creation date/time.
 
-        FIXME: include a property that converts this to a Python ``datetime``.
+        :ref:`uproot.reading.ReadOnlyDirectory.created_on` presents this time
+        as a Python datetime.
         """
         return self._fDatimeC
 
@@ -2065,7 +2102,8 @@ class ReadOnlyDirectory(Mapping):
         """
         Raw integer date/time of last modification.
 
-        FIXME: include a property that converts this to a Python ``datetime``.
+        :ref:`uproot.reading.ReadOnlyDirectory.modified_on` presents this time
+        as a Python datetime.
         """
         return self._fDatimeM
 
@@ -2383,6 +2421,16 @@ class ReadOnlyKey(object):
         """
         return self._fNbytes - self._fKeylen
 
+    @property
+    def created_on(self):
+        """
+        Creation date/time as a Python datetime.
+
+        :doc:`uproot.reading.ReadOnlyKey.fDatime` presents this in ROOT's
+        raw integer encoding.
+        """
+        return uproot._util.code_to_datetime(self._fDatime)
+
     def get(self):
         """
         Returns the object pointed to by this ``TKey``, decompressing it if
@@ -2529,7 +2577,8 @@ class ReadOnlyKey(object):
         """
         Raw integer date/time when the object was written.
 
-        FIXME: include a property that converts this to a Python ``datetime``.
+        :doc:`uproot.reading.ReadOnlyKey.created_on` presents this as a Python
+        datetime.
         """
         return self._fDatime
 
